"""University departments.
"""
import grok
from zope.component import createObject
from zope.component.interfaces import IFactory
from zope.interface import implementedBy
from waeup.interfaces import IDepartment
from waeup.widgets.interfaces import ITableProvider
from waeup.viewlets import MainArea, LeftSidebar, Index, FormWrapMixin, Manage

class Department(grok.Container):
    grok.implements(IDepartment)

    # A simple counter for ids.
    next_id = 0L
    
    def __init__(self,
                 title=u'Unnamed Department',
                 title_prefix=u'department',
                 code=u"NA", **kw):
        super(Department, self).__init__(**kw)
        self.title = title
        self.title_prefix = title_prefix
        self.code = code
        self.courses = createObject(u'waeup.CourseContainer')
        self.courses.__parent__ = self
        self.courses.__name__ = 'courses'
        self.certificates = createObject(u'waeup.CertificateContainer')
        self.certificates.__parent__ = self
        self.certificates.__name__ = 'certificates'

    def traverse(self, name):
        if name == 'courses':
            return self.courses
        elif name == 'certificates':
            return self.certificates
        return None
        

class DepartmentFactory(grok.GlobalUtility):
    """A factory for department containers.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Department')
    title = u"Create a new department.",
    description = u"This factory instantiates new department instances."

    def __call__(self, *args, **kw):
        return Department(*args, **kw)

    def getInterfaces(self):
        return implementedBy(Department)

        
class DisplayForm(grok.DisplayForm):
    grok.context(IDepartment)
    form_fields = grok.AutoFields(IDepartment)

class Content(FormWrapMixin, grok.Viewlet):
    """A viewlet that wraps the `AddDepartmentForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IDepartment)
    grok.view(Index)
    grok.require('waeup.manageUniversity')

    formview_name = 'displayform' # The name of the formview we want
                                  # to be rendered in this viewlet.


class CourseList(grok.Viewlet):
    grok.viewletmanager(MainArea)
    grok.context(IDepartment)
    grok.view(Index)
    grok.order(3)

    def update(self):
        self.table = ITableProvider(self.context).getTable(view=self.view)
        self.table.need()


class ManageForm(grok.EditForm):
    """Manage the basic properties of a `Department` instance.
    """

    form_fields = grok.AutoFields(IDepartment)

    @property
    def label(self):
        # Set department name as form label
        return self.form_fields['title'].field.get(self.context)
    
    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class ManageDepartment(FormWrapMixin, grok.Viewlet):
    """A viewlet that renders the `ManageForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IDepartment)
    grok.view(Manage)
    grok.require('waeup.manageUniversity')
    
    formview_name = 'manageform' # The name of the formview we want to
                                 # be rendered in this viewlet.

class ManageLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IDepartment)
    grok.view(Index)
    grok.order(5)
    # This link is only displayed, if the user is allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="manage">Settings</a></div>'
